<?php

/**
 * 该文件由Xauth系统维护
 * 
 * 生效步骤：
 * 1. 在composer.json中，在"autoload/files"中添加本项目："app/Helpers.php"
 * 2. 在当前项目命令行下执行: composer dumpautoload 
 * 
 * @version 20231212
 * 1.新增了valid_keys()函数
 * 为了让数据表对应的验证器仅在数据表所在项目中撰写，并在其他子项目中通用，该函数用于对验证器中的验证参数有选择地检测和运行。 
 * 2.新增了valid_key()函数
 * 用于验证某个验证参数
 * @version 20231209
 * 1.新增了getDevice($req) 函数
 * 如果指定了device参数，则以有效的参数指定的设备为准；否则，系统自动判断当前使用的设备
 * @version 20230722
 * 1.改进了querykeywords，使keywords参数可以接受null值。
 * @version 20230712
 * 1.更新了Msg类
 * 2.更新了querykeywords类
 * @version 20230711
 * 1.新增了GResponse类
 * 2.删除ValidatesLoads类
 * @version 20230710 
 * 1.新增了从集合根据其值删除元素的函数 rmcolval()
 * 
 * @version 20230709
 * 1.更正了Response类，由`Illuminate\Support\Facades\Response`改为`Illuminate\Http\Response`

 * @version 20230622
 * 1.更新validtable，使用支持defxx_tablename_xx和tablename_xx两种形式。
 * 2.增加了各个子系统的APPID常数定义。
 * @version 20230617x
 * 1.将msgid()和failmsg()函数捆绑进Msg类
 * @version 20230617
 * 1.新增了msgid()函数，构造由appid、tableid、validateindex和localid组成的日志码
 * 2.新增了failmsg()函数，用于取代validstr()函数
 * 3.修改了appid()函数，根据app名称获得其id
 * 4.新增了Msg类
 * @version 20230610
 * 增加了ValidatesLoads类，用于控制器中验证器结果能附带数据。
 * @version 20230128
 * 1.增加了validlogdd($filename,$lineno,$msg,$error_level=E_USER_ERROR)
 * 用于记录系统级日志，并停止执行PHP代码，供开发人员调试。
 * 2.增加了validlogdd_prerequisite($filename,$lineno,$error_level=E_USER_ERROR)
 * 用于记录执行过程中缺少先决条件时的日志，并停止执行PHP代码。
 * 
 * @version 20220828
 * 1.增加了dayofweek_zh(int $dayofweek) 根据数字的星期，返回中文。例如，0=>日，1=>一
 * 
 * @version 20220822 
 * 设置了基础函数。
 * 
 * 函数列表：
 * 1.flat2tree(Collection $flat, $fid = null, string $childrenname = 'items') 
 *   将平面记录转换成dhxtree的嵌套结构。
 * 2.querykeywords($where, array $fields, ?string $keywords) 
 *   构造关键字对不同字段的检索条件语句。
 * 3.dayofweek_zh(int $dayofweek) 
 *   根据数字的星期，返回中文。例如，0=>日，1=>一
 * 4.validstr($tableid, $msgid, $str) 
 *   返回验证错误信息。（准备废除）
 * 5.validtable($keys, $tablename) 
 *   检查是否包含相应数据表名相关的验证条件。
 * 6.validlogdd($filename,$lineno,$msg,$error_level=E_USER_ERROR)
 *   用于记录系统级日志，并停止执行PHP代码，供开发人员调试。
 * 7.validlogdd_prerequisite($filename,$lineno,$error_level=E_USER_ERROR)
 *   用于记录执行过程中缺少先决条件时的日志，并停止执行PHP代码。
 * 8.appid(?string $appname=null):int
 *   根据app名获得appid。
 * 9.rmcolval(Collection &$collection, int|float|string $val)?Collection
 * 类列表：
 * 1.Msg
 * 2.GResponse
 */

use Illuminate\Http\Request;
use Illuminate\Support\Collection;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Log;
use Jenssegers\Agent\Agent;

if (!function_exists("flat2tree")) {
    /**
     * 将线性集合转成树状集合
     * 
     * @param Collection $flag 线性集合。有几个条件：
     *  1. 元素必须是对象，必须包含的字段有id、fid；
     *      注：对象可通过带索引数组创建：json_decode(json_encode($array));
     *  2. 键值等同于id。
     * @param String|Integer $fid 父编号。
     * @param String $childrenname 子元素名称。
     *  即，将子节点放在哪个元素下。默认为 items。
     * @return Collection 树状集合。
     * 注意：顶层集合也有可能有多个节点。
     */
    function flat2tree(Collection $flat, $fid = null, String $childrenname = 'items')
    {
        $tree = collect();
        if ($flat->isEmpty()) {
            return $tree;
        }
        $tree = $flat->filter(function ($item) use ($fid) {
            return $item->fid == $fid;
        });
        $flat = $flat->reject(function ($item) use ($fid) {
            return $item->fid == $fid;
        });

        $tree = $tree->map(function ($item) use ($flat, $childrenname) {
            $children = flat2tree($flat, $item->id, $childrenname);
            if (!$children->isEmpty()) {
                $item->$childrenname = $children;
            }
            return $item;
        });

        return $tree->values();
    }
}

if (!function_exists('querykeywords')) {
    /**
     * 根据关键字构建查询语句
     * 
     * @param Query $where 传入的查询对象。
     * @param Array $fields 要比较的字段。例如：['field','tbl.field1']
     * @param String $keywords 关键字。取值长度为0-32，超出部门将被截断。
     *  关键字可提供多个，以空格区分；
     *  同一个关键字在所有字段中至少出现一次，因此，关键字间是“与”的关系，字段间是“或”的关系。
     * @return Query
     *  
     */
    function querykeywords($where, array $fields, ?string $keywords)
    {
        $maxlength = 32;
        $keywords = $keywords ?? '';
        if (strlen($keywords) > $maxlength) {
            $keywords = substr($keywords, 0, $maxlength);
        }
        $keywords = trim($keywords);
        if ($keywords == '') {
            return $where;
        }

        $keywords = collect(explode(' ', $keywords))
            ->each(function ($keyword) {
                $keyword = trim($keyword);
            })
            ->filter(function ($keyword) {
                return strlen($keyword) > 0;
            })
            ->map(function ($keyword) {
                return '%' . $keyword . '%';
            });

        foreach ($keywords as $keyword) {
            $where = $where->where(function ($query) use ($fields, $keyword) {
                foreach ($fields as $field) {
                    $query->orWhere($field, 'LIKE', $keyword);
                }
            });
        }

        return $where;
    }
}
if (!function_exists('dayofweek_zh')) {
    function dayofweek_zh(int $dayofweek)
    {
        $weekday = '';
        switch ($dayofweek) {
            case 0:
                $weekday = '日';
                break;
            case 1:
                $weekday = '一';
                break;
            case 2:
                $weekday = '二';
                break;
            case 3:
                $weekday = '三';
                break;
            case 4:
                $weekday = '四';
                break;
            case 5:
                $weekday = '五';
                break;
            case 6:
                $weekday = '六';
                break;
        }
        return $weekday;
    }
}
if (!function_exists('validstr')) {
    /**
     * 构造Laravel中的验证器$fail()处理的文本，即："$tableid$msgid|msg"的形式
     * 
     * @param Integer $tableid 数据表编号，以便定位发生的数据表。
     * @param Integer $msgid 日志码编号。
     * @param String $str 日志文本。
     * @return String 返回构造后的文本。
     */
    function validstr($tableid, $msgid, $str)
    {
        return sprintf('%d%03d|%s', $tableid, $msgid, $str);
    }
}
if (!function_exists('validtable')) {
    /**
     * 检查集合中是否包含给定数据表名
     * 
     * 集合元素是定义的验证条件，主要包含2或3部分，形如：tablename###_XXXXX 或 XXXXX_tablename###_XXXXX
     * 其中，
     *      ### 表示日志编号
     *      tablename 是数据表名。每个验证条件都包含了其所在的数据表名，也即这里要检测的表名
     *      XXXX 是具体的日志信息
     * @param Collection $keys 
     * @param String $tablename
     * @return Boolean 是否包含表名。true｜false
     * @deprecated 键值形式请使用 tablename###_XXXXXX 的形式，例如：tags1_AdminuuidMe_is
     */
    function validtable(Collection $keys, String $tablename)
    {
        return !is_null($keys->first(function ($key) use ($tablename) {

            list($t1, $t2,) = explode('_', $key);
            return $t1 == $tablename || $t2 == $tablename;
        }));
    }
}
if (!function_exists('validlogdd')) {
    /**
     * @deprecated 即将废除
     */
    function validlogdd(String $file, int $line, String $msg, $error_level = E_USER_ERROR)
    {
        $str = sprintf("文件(%s)第%d行：%s \n", $file, $line, $msg);
        trigger_error($str, $error_level);
        die();
    }
}
if (!function_exists('validlogdd_prerequisite')) {
    /**
     * @deprecated 即将废除
     */
    function validlogdd_prerequisite(String $file, int $line, $error_level = E_USER_ERROR)
    {
        $str = '缺少先决执行条件！';
        validlogdd($file, $line, $str, $error_level);
    }
}
if (!function_exists('valid_keys')) {
    /**
     * 判断给出的验证参数在当前项目中是否可用
     * 
     * 如果不可用，则程序将中止。
     * 
     * @param array $appnames 项目名数组，如 ['Xauth','Org']。表示要验证的参数只在该项目中有效。
     * @param array $keys_invalid 要验证的参数，这些参数只在上述指定项目中有效，在其他项目中无效。
     * @param String 出错的文件名
     * @param int 出错的代码行
     * 
     * @return 无
     */
    function valid_keys(array $appnames = [], Collection $keys, array $keys_invalid = [], String $pathfile = '', int $line = 0)
    {
        if (!in_array(env('APP_NAME'), $appnames)) {
            $k = $keys->intersect($keys_invalid);
            if ($k->isNotEmpty()) {
                $str = sprintf(
                    '在项目%s中使用了不支持的验证方法[%s]（%s，第%d行）',
                    env('APP_NAME'),
                    $k->implode(', '),
                    $pathfile,
                    $line
                );
                Log::error($str);
                exit;
            }
        }
    }
}
if (!function_exists('valid_key')) {
    /**
     * 验证某个参数
     * 
     * @param int $errno 验证错误编号
     * @param string $key 当前验证的参数名 
     * @param string $fulltableid 验证错误所处的完整数据表编号
     * @param Collection &$keys 待验证的参数集合
     * @param int $vid 验证器局部编号
     * @param Mix $paras 用作闭包函数 callback的参数 
     * @param Callback $callback 检测函数，验证正确返回true，否则返回false
     * @param string $msg_positive 正面错误信息 
     * @param string $msg_negative 反面错误信息，默认null
     * @return GResponse 若通过，则方法 issuccess()为true；否则为false，并返回 Msg $msg 
     */
    function valid_key(int $errno, string $key, string $fulltableid, $paras, Collection &$keys, int $vid, $callback, $msg_positive, $msg_negative = null): GResponse
    {
        if (!$keys->contains($key)) return (new GResponse(response(['type' => 'success']))); // 该键不在检测范围，通过验证

        rmcolval($keys, $key);
        $msg = new Msg($errno, $vid, $fulltableid, $msg_positive, $msg_negative);
        if (!$callback($paras)) {
            return (new GResponse(response(['type' => 'error'])))
                ->setLoad('msg', $msg);
        }
        Log::info($msg->getMsgid() . $msg->getMsg());
        return (new GResponse(response(['type' => 'success'])));
    }
}

if (!function_exists('appid')) {
    define('APPID_ORG', 2);
    define('APPID_SUPERVISE', 3);
    define('APPID_RESMGR', 5);
    define('APPID_SEATMGR', 6);
    define('APPID_BRRMGR', 7);
    define('APPID_ROLLCALL', 8);
    define('APPID_KPIMGR', 9);
    define('APPID_ALUMNI', 10);
    define('APPID_RACKMGR', 11);
    define('APPID_ALGODEMO', 12);
    define('APPID_DPMGR', 13);
    define('APPID_KEYBOX', 14);
    define('APPID_OLQUIZ', 15);
    define('APPID_PGGANTT', 16);
    define('APPID_CROWDSENSOR', 17);
    define('APPID_DOCS', 18);
    define('APPID_XAUTH', 19);
    define('APPID_BRBS', 20);
    function appid(?string $appname = null): int
    {
        if (!$appname || strlen($appname) == 0) {
            return env('APP_ID');
        }
        $appname = strtoupper($appname);
        switch ($appname) {
            case 'ORG':
                return APPID_ORG;
            case 'SUPERVISE':
                return APPID_SUPERVISE;
            case 'RESMGR':
                return APPID_RESMGR;
            case 'SEATMGR':
                return APPID_SEATMGR;
            case 'BRRMGR':
                return APPID_BRRMGR;
            case 'ROLLCALL':
                return APPID_ROLLCALL;
            case 'KPIMGR':
                return APPID_KPIMGR;
            case 'ALUMNI':
                return APPID_ALUMNI;
            case 'RACKMGR':
                return APPID_RACKMGR;
            case 'ALGODEMO':
                return APPID_ALGODEMO;
            case 'DPMGR':
                return APPID_DPMGR;
            case 'KEYBOX':
                return APPID_KEYBOX;
            case 'OLQUIZ':
                return APPID_OLQUIZ;
            case 'PGGANTT':
                return APPID_PGGANTT;
            case 'CROWDSENSOR':
                return APPID_CROWDSENSOR;
            case 'DOCS':
                return APPID_DOCS;
            case 'XAUTH':
                return APPID_XAUTH;
            case 'BRBS':
                return APPID_BRBS;
            default:
                validlogdd(__FILE__, __LINE__, '不存在的应用程序编号！');
        }
    }
}
if (!function_exists('rmcolval')) {
    function rmcolval(Collection &$collection, string|int|float $val): ?Collection
    {
        $flag = $collection->search($val);
        if ($flag === false) {
            return null;
        }
        $collection->forget($flag);
        return $collection;
    }
}
if (!function_exists('getDevice')) {
    function getDevice(Request $req)
    {

        if ($req->has('device')) {
            if (in_array($req->device, [
                'phone',
            ])) {
                $device = 'phone';
            } else {
                $device = 'pc';
            }
        } else {
            $agent = new Agent();
            $device = $agent->isPhone() ? 'phone' : 'pc';
        }

        return $device;
    }
}
class Msg
{
    private ?string $msg, $umsg;
    private ?int
        $fulltableid,
        /**
         * 验证序号：
         * 1 - 验证单数记录，例如：validate_tag
         * 2 - 验证复数记录，例如：validate_tags
         * 11 - 根据id获得单数记录，例如：validate_tagById
         * 21 - 根据ids获得复数记录，例如：validate_tagsByIds
         * 51 - 验证id，例如：validate_tags_id
         * 71 - 验证ids，例如：validate_tags_ids
         */
        $vid,
        /**
         * 局部日志码，
         * 对于验证记录而言：
         *  10-14 验证相关人员是否为我
         *  20-29 验证是否启用
         *  30-39 父编号相关
         *  100-199  单个枚举值
         *  200-  多个字段联合验证
         * 对于验证获取记录而言，取值为：
         *  10 - 无记录
         *  11 - 单条记录
         *  12 - 存在记录
         *  14 - 全部存在（可选）
         * 对于验证字段，0表示成功，其余错误信息从1开始递增。
         */
        $localid;
    function __construct(int $localid = 0, int $vid = 0, int $fulltableid = 0, ?string $msg = null, ?string $umsg = null)
    {
        $this->localid = $localid;
        $this->vid = $vid;
        $this->fulltableid = $fulltableid;
        $this->setMsg($msg);
        $this->setUmsg($umsg);
    }
    public function setMsg(?string $msg = null): Msg
    {
        $this->msg = $msg;
        return $this;
    }
    public function setUmsg(?string $umsg = null): Msg
    {
        $this->umsg = $umsg;
        return $this;
    }
    public function getMsg(): string
    {
        return $this->msg ? $this->msg : '';
    }
    public function getUmsg(): string
    {
        return $this->umsg ? $this->umsg : '不满足条件：' . $this->getMsg();
    }
    public function getMsgid(): string
    {
        return sprintf('%04d%02d%03d', $this->fulltableid, $this->vid, $this->localid);
    }
    public function getFailmsg(): string
    {
        return sprintf('%09d|%s', $this->getMsgid(), $this->getUmsg());
    }
}

class GResponse
{
    private ?Response $response;
    private ?stdClass $loads;

    public function __construct(?Response $response = null)
    {
        $this->response = $response;
        $this->loads = new stdClass();
    }
    function __invoke(): ?Response
    {
        return $this->response;
    }
    private function isTypeValue(string $val): bool
    {
        if (!$this->response) return false;
        $content = json_decode($this->response->getContent());
        return $content->type == $val;
    }
    function issuccess(): bool
    {
        return $this->isTypeValue('success');
    }
    function iserror(): bool
    {
        return $this->isTypeValue('error');
    }
    function iswarning(): bool
    {
        return $this->isTypeValue('warning');
    }
    public function setResponse(?Response $response)
    {
        $this->response = $response;
        return $this;
    }
    public function getResponse()
    {
        return $this->response;
    }
    public function setLoad(string $loadname, mixed $load): GResponse
    {
        $this->loads->{$loadname} = $load;
        return $this;
    }
    public function getLoad(string $loadname): mixed
    {
        if (!isset($this->loads->{$loadname})) {
            dd('获取不存在的成员变量：' . $loadname);
        }
        return $this->loads->{$loadname};
    }
}
